@@ -256,13 +256,17 @@ try (GVfsJob *job)
   is_busy = g_vfs_daemon_has_blocking_processes (g_vfs_backend_get_daemon (backend));
   force_unmount = op_job->flags & G_MOUNT_UNMOUNT_FORCE;
   
-  if (is_busy && ! force_unmount
-      && ! g_mount_source_is_dummy (op_job->mount_source))
+  if (is_busy && !force_unmount)
     {
-      g_vfs_backend_unmount_with_operation (backend,
-					    op_job->mount_source,
-					    (GAsyncReadyCallback) unmount_cb,
-					    op_job);
+      if (g_mount_source_is_dummy (op_job->mount_source))
+        g_vfs_job_failed_literal (G_VFS_JOB (op_job),
+                                  G_IO_ERROR, G_IO_ERROR_BUSY,
+                                  _("File system is busy"));
+      else
+        g_vfs_backend_unmount_with_operation (backend,
+                                              op_job->mount_source,
+                                              (GAsyncReadyCallback)unmount_cb,
+                                              op_job);
       return TRUE;
     }
 
