@@ -215,134 +215,6 @@ handle_set (GVfsMetadata *object,
   return TRUE;
 }
 
-static void
-append_key (GVariantBuilder *builder,
-	    MetaTree *tree,
-	    const char *path,
-	    const char *key)
-{
-  MetaKeyType keytype;
-  char *str;
-  char **strv;
-
-  keytype = meta_tree_lookup_key_type (tree, path, key);
-
-  if (keytype == META_KEY_TYPE_STRING)
-    {
-      str = meta_tree_lookup_string (tree, path, key);
-      g_variant_builder_add (builder, "{sv}", key, g_variant_new_string (str));
-      g_free (str);
-    }
-  else if (keytype == META_KEY_TYPE_STRINGV)
-    {
-      strv = meta_tree_lookup_stringv (tree, path, key);
-      g_variant_builder_add (builder, "{sv}", key, g_variant_new_strv ((const gchar * const  *) strv, -1));
-      g_strfreev (strv);
-    }
-}
-
-static gboolean
-enum_keys (const char *key,
-	   MetaKeyType type,
-	   gpointer value,
-	   gpointer user_data)
-{
-  GPtrArray *keys = user_data;
-
-  g_ptr_array_add (keys, g_strdup (key));
-  return TRUE;
-}
-
-static gboolean
-handle_get (GVfsMetadata *object,
-            GDBusMethodInvocation *invocation,
-            const gchar *arg_treefile,
-            const gchar *arg_path,
-            const gchar *const *arg_keys,
-            GVfsMetadata *daemon)
-{
-  TreeInfo *info;
-  GPtrArray *meta_keys;
-  gboolean free_keys;
-  gchar **iter_keys;
-  gchar **i;
-  GVariantBuilder *builder;
-
-  info = tree_info_lookup (arg_treefile);
-  if (info == NULL)
-    {
-      g_dbus_method_invocation_return_error (invocation,
-                                             G_IO_ERROR,
-                                             G_IO_ERROR_NOT_FOUND,
-                                             _("Can't find metadata file %s"),
-                                             arg_treefile);
-      return TRUE;
-    }
-
-  if (arg_keys == NULL)
-    {
-      /* Get all keys */
-      free_keys = TRUE;
-      meta_keys = g_ptr_array_new ();
-      meta_tree_enumerate_keys (info->tree, arg_path, enum_keys, meta_keys);
-      iter_keys = (gchar **) g_ptr_array_free (meta_keys, FALSE);
-    }
-  else
-    {
-      free_keys = FALSE;
-      iter_keys = (gchar **) arg_keys;
-    }
-
-  builder = g_variant_builder_new (G_VARIANT_TYPE_VARDICT);
-
-  for (i = iter_keys; *i; i++)
-    append_key (builder, info->tree, arg_path, *i);
-  if (free_keys)
-    g_strfreev (iter_keys);
-  
-  gvfs_metadata_complete_get (object, invocation,
-                              g_variant_builder_end (builder));
-  g_variant_builder_unref (builder);
-
-  return TRUE;
-}
-
-static gboolean
-handle_unset (GVfsMetadata *object,
-              GDBusMethodInvocation *invocation,
-              const gchar *arg_treefile,
-              const gchar *arg_path,
-              const gchar *arg_key,
-              GVfsMetadata *daemon)
-{
-  TreeInfo *info;
-
-  info = tree_info_lookup (arg_treefile);
-  if (info == NULL)
-    {
-      g_dbus_method_invocation_return_error (invocation,
-                                             G_IO_ERROR,
-                                             G_IO_ERROR_NOT_FOUND,
-                                             _("Can't find metadata file %s"),
-                                             arg_treefile);
-      return TRUE;
-    }
-
-  if (!meta_tree_unset (info->tree, arg_path, arg_key))
-    {
-      g_dbus_method_invocation_return_error_literal (invocation,
-                                                     G_IO_ERROR,
-                                                     G_IO_ERROR_FAILED,
-                                                     _("Unable to unset metadata key"));
-      return TRUE;
-    }
-
-  tree_info_schedule_writeout (info);
-  gvfs_metadata_complete_unset (object, invocation);
-
-  return TRUE;
-}
-
 static gboolean
 handle_remove (GVfsMetadata *object,
                GDBusMethodInvocation *invocation,
@@ -418,31 +290,6 @@ handle_move (GVfsMetadata *object,
   return TRUE;
 }
 
-static void
-on_name_acquired (GDBusConnection *connection,
-                  const gchar     *name,
-                  gpointer         user_data)
-{
-  GError *error;
-  
-  skeleton = gvfs_metadata_skeleton_new ();
-  
-  g_signal_connect (skeleton, "handle-set", G_CALLBACK (handle_set), skeleton);
-  g_signal_connect (skeleton, "handle-unset", G_CALLBACK (handle_unset), skeleton);
-  g_signal_connect (skeleton, "handle-get", G_CALLBACK (handle_get), skeleton);
-  g_signal_connect (skeleton, "handle-remove", G_CALLBACK (handle_remove), skeleton);
-  g_signal_connect (skeleton, "handle-move", G_CALLBACK (handle_move), skeleton);
-
-  error = NULL;
-  if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (skeleton), connection,
-                                         G_VFS_DBUS_METADATA_PATH, &error))
-    {
-      g_printerr ("Error exporting volume monitor: %s (%s, %d)\n",
-                  error->message, g_quark_to_string (error->domain), error->code);
-      g_error_free (error);
-    }
-}
-
 static void
 on_name_lost (GDBusConnection *connection,
               const gchar     *name,
@@ -525,8 +372,6 @@ main (int argc, char *argv[])
       return 0;
     }
 
-  loop = g_main_loop_new (NULL, FALSE);
-
   error = NULL;
   conn = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
   if (!conn)
@@ -542,6 +387,7 @@ main (int argc, char *argv[])
 				      NULL,
 				      (GDestroyNotify)tree_info_free);
 
+  loop = g_main_loop_new (NULL, FALSE);
   g_dbus_connection_set_exit_on_close (conn, FALSE);
   g_signal_connect (conn, "closed", G_CALLBACK (on_connection_closed), loop);
 
@@ -549,10 +395,28 @@ main (int argc, char *argv[])
   if (replace)
     flags |= G_BUS_NAME_OWNER_FLAGS_REPLACE;
 
+  skeleton = gvfs_metadata_skeleton_new ();
+
+  g_signal_connect (skeleton, "handle-set", G_CALLBACK (handle_set), skeleton);
+  g_signal_connect (skeleton, "handle-remove", G_CALLBACK (handle_remove), skeleton);
+  g_signal_connect (skeleton, "handle-move", G_CALLBACK (handle_move), skeleton);
+
+  error = NULL;
+  if (!g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (skeleton), conn,
+                                         G_VFS_DBUS_METADATA_PATH, &error))
+    {
+      g_printerr ("Error exporting metadata daemon: %s (%s, %d)\n",
+                  error->message, g_quark_to_string (error->domain), error->code);
+      g_error_free (error);
+      g_object_unref (conn);
+      g_main_loop_unref (loop);
+      return 1;
+    }
+
   name_owner_id = g_bus_own_name_on_connection (conn,
                                                 G_VFS_DBUS_METADATA_NAME,
                                                 flags,
-                                                on_name_acquired,
+                                                NULL,
                                                 on_name_lost,
                                                 loop,
                                                 NULL);
